﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
sys.path.append('/usr/lib/openoffice/basis-link/program')
sys.path.append('/usr/lib/libreoffice/basis-link/program')
import os
import uno
import string
import psycopg2
import time

# a UNO struct later needed to create a document
from com.sun.star.text.ControlCharacter import PARAGRAPH_BREAK
from com.sun.star.text.TextContentAnchorType import AS_CHARACTER
from com.sun.star.awt import Size
from com.sun.star.lang import XMain
from com.sun.star.table import BorderLine
from com.sun.star.table import  XTableRows
import datetime
from time import gmtime, strftime , mktime, localtime

def rgbColor( nRed, nGreen, nBlue ):
    	"""Return an integer which repsents a color.
    	The color is specified in RGB notation.
    	Each of nRed, nGreen and nBlue must be a number from 0 to 255.    """
    	return (int( nRed ) & 255) << 16 | (int( nGreen ) & 255) << 8 | (int( nBlue ) & 255)

def makeSize( nWidth, nHeight ):
    	"""Create a com.sun.star.awt.Size struct."""
    	oSize = createUnoStruct( "com.sun.star.awt.Size" )
    	oSize.Width = nWidth
    	oSize.Height = nHeight
    	return oSize

goServiceManager = False
def getServiceManager( cHost="localhost", cPort="2002" ):
    	"""Get the ServiceManager from the running OpenOffice.org.
        		Then retain it in the global variable goServiceManager for future use.
        		This is similar to the GetProcessServiceManager() in OOo Basic.    """
    	global goServiceManager
    	if not goServiceManager:
        		oLocalContext = uno.getComponentContext()
        		try:
            			oLocalResolver = oLocalContext.ServiceManager.createInstanceWithContext("com.sun.star.bridge.UnoUrlResolver", oLocalContext )
            			oContext = oLocalResolver.resolve( "uno:socket,host=" + cHost + ",port=" + cPort + ";urp;StarOffice.ComponentContext" )
            			goServiceManager = oContext.ServiceManager
        		except:
			#            print "Failed connection to the OOo"
            			sys.exit()           
    	return goServiceManager

def createUnoService( cClass ):
    	"""A handy way to create a global objects within the running OOo.
    	Similar to the function of the same name in OOo Basic.    """
    	oServiceManager = getServiceManager()
    	oObj = oServiceManager.createInstance( cClass )
    	return oObj

def createUnoStruct( cTypeName ):
    	"""Create a UNO struct and return it.
    	Similar to the function of the same name in OOo Basic.    """
    	oCoreReflection = getCoreReflection()
    	oXIdlClass = oCoreReflection.forName( cTypeName )
    	oReturnValue, oStruct = oXIdlClass.createObject( None )
    	return oStruct

goCoreReflection = False
def getCoreReflection():
    	global goCoreReflection
    	if not goCoreReflection:
        		goCoreReflection = createUnoService( "com.sun.star.reflection.CoreReflection" )
    	return goCoreReflection

def makePoint( nX, nY ):
    	"""Create a com.sun.star.awt.Point struct."""
    	oPoint = createUnoStruct( "com.sun.star.awt.Point" )
    	oPoint.X = nX
    	oPoint.Y = nY
    	return oPoint

def insertFormattedText(text, cursor, color, height, weight, font, shadow, alignment, rotation,italic, string):
	cursor.setPropertyValue("CharColor", color)
	cursor.setPropertyValue("CharShadowed", shadow)
	cursor.setPropertyValue("CharHeight", height)
	cursor.setPropertyValue("CharFontName", font)
	cursor.setPropertyValue("CharWeight", weight)
	cursor.setPropertyValue("CharRotation",rotation)
	cursor.setPropertyValue("ParaAdjust",alignment);
	cursor.setPropertyValue ("CharPosture", italic)
	text.insertString(cursor, string,0)

def makeTextFrameWithGraph( oDocument, Width=None , oPosition=None):
    	oFrame = oDocument.createInstance( "com.sun.star.text.TextFrame" )
    	oFrame.Width = Width
    	oFrame.FrameIsAutomaticHeight =1    
    	oFrame.setPropertyValue("VertOrient",None)
    	oFrame.setPropertyValue("HoriOrient",None)
    	oFrame.setPropertyValue("HoriOrientPosition", oPosition.X)
    	oFrame.setPropertyValue("VertOrientPosition", oPosition.Y)  
    	oDocument.Text.insertTextContent( oDocument.Text.Start, oFrame, 0 )    
    	line = oFrame.TopBorder
   	line.InnerLineWidth = line.OuterLineWidth = 0
   	line.Color = rgbColor(255, 255, 255)
    	oFrame.TopBorder = oFrame.BottomBorder= oFrame.LeftBorder = oFrame.RightBorder = line
   	return oFrame

def insertGraphIntoFrame(oDocument, oFrame, oGraphName,  oGraphSize, oPath):
    	text = oFrame.Text
    	cursor = text.createTextCursor()
    	oGraph = oDocument.createInstance("com.sun.star.text.GraphicObject")
    	dir = os.path.dirname(__file__)    
    	oGraph.GraphicURL = "file:///" + oPath + "/" + oGraphName
    	oGraph.Size = oGraphSize
    	text.insertTextContent( cursor, oGraph, 0 )

def insertTextIntoFrame(oFrame, string, color, alignment, CharHeight,  CharWeight,CharUnderline,  CharFontName):
    	text = oFrame.Text
    	cursor = text.createTextCursor()
    	cursor.gotoEnd(0)
    	cursor.setPropertyValue("CharColor", color)
    	cursor.setPropertyValue("CharRotation",0)
    	cursor.setPropertyValue("ParaAdjust",alignment);
    	cursor.setPropertyValue("CharHeight", CharHeight)
    	cursor.setPropertyValue("CharFontName", CharFontName)
    	cursor.setPropertyValue("CharUnderline", CharUnderline)
    	cursor.setPropertyValue("CharWeight", CharWeight) 
    	cursor.setPropertyValue("CharKerning", 0) 
    	text.insertString(cursor,string, 0 )

def insertGerb(Document, oPath):
    	oFrame = makeTextFrameWithGraph( Document,  8000 , makePoint(400,1000))    
    	insertGraphIntoFrame(Document, oFrame, "gerb.png", makeSize(1500, 1900),oPath)    
   	
    	insertTextIntoFrame(oFrame, "\r"+"\r"+"\r"+"\r"+"МЧС РОССИИ".decode("utf-8") +"\r", 0, 3, 10, 150, 1,  "Times New Roman")
    	insertTextIntoFrame(oFrame, "Главное управление ".decode("utf-8") +"\n", 0, 3, 10, 150, 0,  "Times New Roman")
       	insertTextIntoFrame(oFrame, "Министерства Российской Федерации ".decode("utf-8")+"\n", 0, 3, 10, 150, 0,  "Times New Roman")
    	insertTextIntoFrame(oFrame, "по делам гражданской обороны,".decode("utf-8")+"\n", 0, 3, 10, 150, 0,  "Times New Roman")    
    	insertTextIntoFrame(oFrame, "чрезвычайным ситуациям и ликвидации".decode("utf-8")+"\n", 0, 3, 10, 150, 0,  "Times New Roman")
    	insertTextIntoFrame(oFrame, "последствий стихийных бедствий".decode("utf-8")+"\n", 0, 3, 10, 150, 0,  "Times New Roman")
    	insertTextIntoFrame(oFrame, "По                        области".decode("utf-8")+"\n", 0, 3, 10, 150, 0,  "Times New Roman")
    	insertTextIntoFrame(oFrame, ""+"\n", 0, 3, 10, 150, 0,  "Times New Roman")
    	#string = "По Омской области)\n"
    	insertTextIntoFrame(oFrame, ""+"\n", 0, 3, 10, 150, 0,  "Times New Roman")
    	#string = "Интернациональная ул., 41, Омск, 644043 \n"
    	insertTextIntoFrame(oFrame, ""+"\n", 0, 3, 10, 0, 0,  "Times New Roman")
    	#string = "Тел: 25-65-16, факс: 25-54-16 \n"
    	insertTextIntoFrame(oFrame, ""+"\n", 0, 3, 10, 0, 0,  "Times New Roman")
    	#string = "E-mail: kia@ugps.omsk.su \n"
    	insertTextIntoFrame(oFrame, ""+"\n", 0, 3, 10, 0, 0,  "Times New Roman")
    	#string = "%sг."%strftime("%d.%m.%Y", gmtime()) 
    	insertTextIntoFrame(oFrame, "", 0, 3, 10, 0, 0,  "Times New Roman")

def insertRightFrame1(Document):
    	oFrame = makeTextFrameWithGraph( Document,  7000 , makePoint(10000,2200)) 
    	line = oFrame.TopBorder 
    	line.OuterLineWidth = 10
    	line.InnerLineWidth = 0
    	line.Color = rgbColor(0, 0, 0)
    	oFrame.TopBorder = oFrame.BottomBorder= oFrame.LeftBorder = oFrame.RightBorder = line
    	insertTextIntoFrame(oFrame, "\"ЦУКС СРЦ\"".decode("utf-8")+"\n", 0, 3, 13, 0, 0,  "Times New Roman")
    	insertTextIntoFrame(oFrame, "Старшему оперативному".decode("utf-8")+"\n", 0, 3, 13, 0, 0,  "Times New Roman")
    	insertTextIntoFrame(oFrame, "дежурному".decode("utf-8"), 0, 3, 13, 0, 0,  "Times New Roman")
    	



