#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Python libraries

import math
import string
import sys
import time
sys.path.append('/usr/lib/libreoffice/basis3.2/program')
sys.path.append('/usr/lib/openoffice/basis3.2/program')

# Danny's libraries
#from Danny import HSBConversions
#from Danny.OOo.OOoLib import *

# OOo's libraries
import uno
# a UNO struct later needed to create a document
from com.sun.star.text.ControlCharacter import PARAGRAPH_BREAK
from com.sun.star.text.TextContentAnchorType import AS_CHARACTER
from com.sun.star.awt import Size
from com.sun.star.lang import XMain
from com.sun.star.table import BorderLine


#интерфейсы для таблиц
from  com.sun.star.table import XCell;
from  com.sun.star.table import XCellRange;
from  com.sun.star.table import XTable;


#------------------------------------------------------------
#   Functions for working with Draw documents.
#------------------------------------------------------------
def makeDrawDocument():
    """Create a new OOo Draw document."""
    return loadComponentFromURL( "private:factory/sdraw" )

def makeImpressDocument():
    """Create a new OOo Impress document."""
    return loadComponentFromURL( "private:factory/simpress" )

# Notes about some properties and constants for shape objects...

    # LineStyle can be one of...
    #   com.sun.star.drawing.LineStyle.NONE
    #   com.sun.star.drawing.LineStyle.SOLID
    #   com.sun.star.drawing.LineStyle.DASH

    # CircleKind can be one of...
    #   com.sun.star.drawing.CircleKind.FULL
    #   com.sun.star.drawing.CircleKind.SECTION   ' a circle with a cut connected by two lines
    #   com.sun.star.drawing.CircleKind.CUT ' a circle with a cut connected by a line
    #   com.sun.star.drawing.CircleKind.ARC ' a circle with an open cut

    # FillStyle can be one of...
    #   com.sun.star.drawing.FillStyle.NONE
    #   com.sun.star.drawing.FillStyle.SOLID
    #   com.sun.star.drawing.FillStyle.GRADIENT
    #   com.sun.star.drawing.FillStyle.HATCH
    #   com.sun.star.drawing.FillStyle.BITMAP

    # TextHorizontalAdjust can be one of...
    #   com.sun.star.drawing.TextHorizontalAdjust.LEFT
    #   com.sun.star.drawing.TextHorizontalAdjust.CENTER
    #   com.sun.star.drawing.TextHorizontalAdjust.RIGHT
    #   com.sun.star.drawing.TextHorizontalAdjust.BLOCK

    # TextVerticalAdjust can be one of...
    #   com.sun.star.drawing.TextVerticalAdjust.TOP
    #   com.sun.star.drawing.TextVerticalAdjust.CENTER
    #   com.sun.star.drawing.TextVerticalAdjust.BOTTOM
    #   com.sun.star.drawing.TextVerticalAdjust.BLOCK

    # TextFitToSize can be one of...
    #    com.sun.star.drawing.TextFitToSizeType.NONE
    #    com.sun.star.drawing.TextFitToSizeType.PROPORTIONAL
    #    com.sun.star.drawing.TextFitToSizeType.ALLLINES
    #    com.sun.star.drawing.TextFitToSizeType.RESIZEATTR

# Useful code snippets...

    # Accessing pages of a drawing document.
    #   oDrawPage = oDrawDoc.getDrawPages().getByIndex( 0 )
    #   oDrawPage = oDrawDoc.getDrawPages().getCount()
    #   oDrawPage = oDrawDoc.getDrawPages().insertByIndex( 1 )
    
#    (AvailableServiceNames)
#com.sun.star.chart2.data.DataProvider
#com.sun.star.drawing.AppletShape
#com.sun.star.drawing.CaptionShape
#com.sun.star.drawing.ClosedBezierShape
#com.sun.star.drawing.ClosedFreeHandShape
#com.sun.star.drawing.ConnectorShape
#com.sun.star.drawing.ControlShape
#com.sun.star.drawing.CustomShape
#com.sun.star.drawing.EllipseShape
#com.sun.star.drawing.FrameShape
#com.sun.star.drawing.GraphicObjectShape
#com.sun.star.drawing.GroupShape
#com.sun.star.drawing.LineShape
#com.sun.star.drawing.MeasureShape
#com.sun.star.drawing.MediaShape
#com.sun.star.drawing.OpenBezierShape
#com.sun.star.drawing.OpenFreeHandShape
#com.sun.star.drawing.PageShape
#com.sun.star.drawing.PluginShape
#com.sun.star.drawing.PolyLinePathShape
#com.sun.star.drawing.PolyLineShape
#com.sun.star.drawing.PolyPolygonPathShape
#com.sun.star.drawing.PolyPolygonShape
#com.sun.star.drawing.RectangleShape
#com.sun.star.drawing.Shape3DCubeObject
#com.sun.star.drawing.Shape3DExtrudeObject
#com.sun.star.drawing.Shape3DLatheObject
#com.sun.star.drawing.Shape3DPolygonObject
#com.sun.star.drawing.Shape3DSceneObject
#com.sun.star.drawing.Shape3DSphereObject
#com.sun.star.drawing.TextShape
#---------------------------------------------
goServiceManager = False
def getServiceManager( cHost="localhost", cPort="2002" ):
    """Get the ServiceManager from the running OpenOffice.org.
        Then retain it in the global variable goServiceManager for future use.
        This is similar to the GetProcessServiceManager() in OOo Basic.
    """
    global goServiceManager
    if not goServiceManager:
        # Get the uno component context from the PyUNO runtime
        oLocalContext = uno.getComponentContext()
        # Create the UnoUrlResolver on the Python side.
        oLocalResolver = oLocalContext.ServiceManager.createInstanceWithContext(
                                    "com.sun.star.bridge.UnoUrlResolver", oLocalContext )
        # Connect to the running OpenOffice.org and get its context.
        oContext = oLocalResolver.resolve( "uno:socket,host=" + cHost + ",port=" + cPort + ";urp;StarOffice.ComponentContext" )
        # Get the ServiceManager object
        goServiceManager = oContext.ServiceManager
    return goServiceManager


#------------------------------------------------------------
#   Uno convenience functions
#   The stuff in this section is just to make
#    python progrmaming of OOo more like using OOo Basic.
#------------------------------------------------------------


# This is the same as ServiceManager.createInstance( ... )
def createUnoService( cClass ):
    """A handy way to create a global objects within the running OOo.
    Similar to the function of the same name in OOo Basic.
    """
    oServiceManager = getServiceManager()
    oObj = oServiceManager.createInstance( cClass )
    return oObj

# The StarDesktop object.  (global like in OOo Basic)
# It is cached in a global variable.
StarDesktop = None
def getDesktop():
    """An easy way to obtain the Desktop object from a running OOo.
    """
    global StarDesktop
    if StarDesktop == None:
        StarDesktop = createUnoService( "com.sun.star.frame.Desktop" )
    return StarDesktop
# preload the StarDesktop variable.
getDesktop()

# The CoreReflection object.
# It is cached in a global variable.
goCoreReflection = False
def getCoreReflection():
    global goCoreReflection
    if not goCoreReflection:
        goCoreReflection = createUnoService( "com.sun.star.reflection.CoreReflection" )
    return goCoreReflection


def createUnoStruct( cTypeName ):
    """Create a UNO struct and return it.
    Similar to the function of the same name in OOo Basic.
    """
    oCoreReflection = getCoreReflection()
    # Get the IDL class for the type name
    oXIdlClass = oCoreReflection.forName( cTypeName )
    # Create the struct.
    oReturnValue, oStruct = oXIdlClass.createObject( None )
    return oStruct

#def newConnectionToOOo( cHost="localhost", cPort="8100" ):
#    """Call this to establish, or re-establish a connection to OOo."""
#    global goServiceManager
#    global StarDesktop
#    global goCoreReflection
#    goServiceManager = False
#    StarDesktop = None
#    goCoreReflection = False
#    getServiceManager( cHost, cPort )
#    getDesktop()

#------------------------------------------------------------
#   API helpers
#------------------------------------------------------------

def hasUnoInterface( oObject, cInterfaceName ):
    """Similar to Basic's HasUnoInterfaces() function, but singular not plural."""

    # Get the Introspection service.
    oIntrospection = createUnoService( "com.sun.star.beans.Introspection" )

    # Now inspect the object to learn about it.   
    oObjInfo = oIntrospection.inspect( oObject )

    # Obtain an array describing all methods of the object.
    oMethods = oObjInfo.getMethods( uno.getConstantByName( "com.sun.star.beans.MethodConcept.ALL" ) )
    # Now look at every method.
    for oMethod in oMethods:
        # Check the method's interface to see if
        #  these aren't the droids you're looking for.
        cMethodInterfaceName = oMethod.getDeclaringClass().getName()
        if cMethodInterfaceName == cInterfaceName:
            return True
    return False

def hasUnoInterfaces( oObject, *cInterfaces ):
    """Similar to the function of the same name in OOo Basic."""
    for cInterface in cInterfaces:
        if not hasUnoInterface( oObject, cInterface ):
            return False
    return True

#------------------------------------------------------------
#   High level general purpose functions
#------------------------------------------------------------
def makePropertyValue( cName=None, uValue=None, nHandle=None, nState=None ):
    """Create a com.sun.star.beans.PropertyValue struct and return it.
    """
    oPropertyValue = createUnoStruct( "com.sun.star.beans.PropertyValue" )

    if cName != None:
        oPropertyValue.Name = cName
    if uValue != None:
        oPropertyValue.Value = uValue
    if nHandle != None:
        oPropertyValue.Handle = nHandle
    if nState != None:
        oPropertyValue.State = nState

    return oPropertyValue

def makePoint( nX, nY ):
    """Create a com.sun.star.awt.Point struct."""
    oPoint = createUnoStruct( "com.sun.star.awt.Point" )
    oPoint.X = nX
    oPoint.Y = nY
    return oPoint

def makeSize( nWidth, nHeight ):
    """Create a com.sun.star.awt.Size struct."""
    oSize = createUnoStruct( "com.sun.star.awt.Size" )
    oSize.Width = nWidth
    oSize.Height = nHeight
    return oSize

def makeRectangle( nX, nY, nWidth, nHeight ):
    """Create a com.sun.star.awt.Rectangle struct."""
    oRect = createUnoStruct( "com.sun.star.awt.Rectangle" )
    oRect.X = nX
    oRect.Y = nY
    oRect.Width = nWidth
    oRect.Height = nHeight
    return oRect

def Array( *args ):
#def Array( args ):
    """This is just sugar coating so that code from OOoBasic which
    contains the Array() function can work perfectly in python."""
    tArray = ()
    for arg in args:
        tArray += (arg,)
    return tArray
    
def ListToArray( args ):
    """This is just sugar coating so that code from OOoBasic which
    contains the Array() function can work perfectly in python."""
    tArray = ()
    for arg in args:
        tArray += (arg,)
    return tArray

def loadComponentFromURL( cUrl, tProperties=() ):
    """Open or Create a document from it's URL.
    New documents are created from URL's such as:
        private:factory/sdraw
        private:factory/swriter
        private:factory/scalc
        private:factory/simpress
    """
    StarDesktop = getDesktop()
    oDocument = StarDesktop.loadComponentFromURL( cUrl, "_blank", 0, tProperties )
    return oDocument

#def makeWriterDocument():
#    """Create a new OOo Writer document."""
#    return loadComponentFromURL( "private:factory/swriter" )
#
#
#def makeCalcDocument():
#    """Create a new OOo Calc document."""
#    return loadComponentFromURL( "private:factory/scalc" )

#------------------------------------------------------------
#   Styles
#------------------------------------------------------------
def defineStyle( oDrawDoc, cStyleFamily, cStyleName, cParentStyleName=None ):
    """Add a new style to the style catalog if it is not already present.
    This returns the style object so that you can alter its properties.
    """
    oStyleFamily = oDrawDoc.getStyleFamilies().getByName( cStyleFamily )

    # Does the style already exist?
    if oStyleFamily.hasByName( cStyleName ):
        # then get it so we can return it.
        oStyle = oStyleFamily.getByName( cStyleName )
    else:
        # Create new style object.
        oStyle = oDrawDoc.createInstance( "com.sun.star.style.Style" )

        # Set its parent style
        if cParentStyleName != None:
            oStyle.setParentStyle( cParentStyleName )

        # Add the new style to the style family.
        oStyleFamily.insertByName( cStyleName, oStyle )

    return oStyle

def getStyle( oDrawDoc, cStyleFamily, cStyleName ):
    """Lookup and return a style from the document.
    """
    return oDrawDoc.getStyleFamilies().getByName( cStyleFamily ).getByName( cStyleName )


#------------------------------------------------------------
#   General Utility functions
#------------------------------------------------------------

def convertToURL( cPathname ):
    """Convert a Windows or Linux pathname into an OOo URL."""
    if len( cPathname ) > 1:
        if cPathname[1:2] == ":":
            cPathname = "/" + cPathname[0] + "|" + cPathname[2:]
    cPathname = string.replace( cPathname, "\\", "/" )
    cPathname = "file://" + cPathname
    return cPathname 
    #----------------------------------------------
def RGBtoHSB( nRed, nGreen, nBlue ):
    """RGB to HSB color space conversion routine.
    nRed, nGreen and nBlue are all numbers from 0 to 255.
    This routine returns three floating point numbers, nHue, nSaturation, nBrightness.
    nHue, nSaturation and nBrightness are all from 0.0 to 1.0.
    """
    nMin = min( nRed, nGreen, nBlue )
    nMax = max( nRed, nGreen, nBlue )

    if nMin == nMax:
        # Grayscale
        nHue = 0.0
        nSaturation = 0.0
        nBrightness = nMax
    else:
        if nRed == nMin:
            d = nGreen = nBlue
            h = 3.0
        elif nGreen == nMin:
            d = nBlue - nRed
            h = 5.0
        else:
            d = nRed - nGreen
            h = 1.0

        nHue = ( h - ( float( d ) / (nMax - nMin) ) ) / 6.0
        nSaturation = (nMax - nMin) / float( nMax )
        nBrightness = nMax / 255.0

    return nHue, nSaturation, nBrightness

def HSBtoRGB( nHue, nSaturation, nBrightness ):
    """HSB to RGB color space conversion routine.
    nHue, nSaturation and nBrightness are all from 0.0 to 1.0.
    This routine returns three integer numbers, nRed, nGreen, nBlue.
    nRed, nGreen and nBlue are all numbers from 0 to 255.
    """
    # Scale the brightness from a range of 0.0 thru 1.0
    #  to a range of 0.0 thru 255.0
    # Then truncate to an integer.
    nBrightness = int( min( nBrightness * 256.0, 255.0 ) )

    if nSaturation == 0.0:
        # Grayscale because there is no saturation
        nRed = nBrightness
        nGreen = nBrightness
        nBlue = nBrightness
    else:
        # Make hue angle be within a single rotation.
        # If the hue is > 1.0 or < 0.0, then it has
        #  "gone around the color wheel" too many times.
        #  For example, a value of 1.2 means that it has
        #  gone around the wheel 1.2 times, which is really
        #  the same ending angle as 0.2 trips around the wheel.
        # Scale it back to the 0.0 to 1.0 range.
        if nHue > 1.0:
            nHue = nHue - int( nHue )
        elif nHue < 0.0:
            nHue = abs( nHue )
            if nHue > 1.0:
                nHue = nHue - int( nHue )
            nHue = 1.0 - nHue
        # Rescale hue to a range of 0.0 thru 6.0
        nHue = nHue * 6.0
        # Separate hue into int and fractional parts
        iHue = int( nHue )
        fHue = nHue - iHue
        # Is hue even?
        if iHue % 2 == 0:
            fHue = 1.0 - fHue
        #
        m = nBrightness * (1.0 - nSaturation)
        n = nBrightness * (1.0 - (nSaturation * fHue))

        if iHue == 1:
            nRed = n
            nGreen = nBrightness
            nBlue = m
        elif iHue == 2:
            nRed = m
            nGreen = nBrightness
            nBlue = n
        elif iHue == 3:
            nRed = m
            nGreen = n
            nBlue = nBrightness
        elif iHue == 4:
            nRed = n
            nGreen = m
            nBlue = nBrightness
        elif iHue == 5:
            nRed = nBrightness
            nGreen = m
            nBlue = n
        else:
            nRed = nBrightness
            nGreen = n
            nBlue = m

    return nRed, nGreen, nBlue 
    
#------------------------------------------------------------
#   Document functions
#------------------------------------------------------------

def setDrawPageOrientationLandscape( oDrawPage ):
    """Pass in any GenericDrawPage object, and this changes it to landscape orientation,
     in addition to swapping the height/width as you would expect.
    """
    # Save some settings
    nOldWidth = oDrawPage.Width
    nOldHeight = oDrawPage.Height
    nOldBorderTop = oDrawPage.BorderTop
    nOldBorderLeft = oDrawPage.BorderLeft
    nOldBorderRight = oDrawPage.BorderRight
    nOldBorderBottom = oDrawPage.BorderBottom

    # Change so that it will PRINT in landscape
    oDrawPage.Orientation = uno.getConstantByName( "com.sun.star.view.PaperOrientation.LANDSCAPE" )

    # Now change some paper dimensions to match
    oDrawPage.Width = nOldHeight
    oDrawPage.Height = nOldWidth
    oDrawPage.BorderTop = nOldBorderRight
    oDrawPage.BorderLeft = nOldBorderTop
    oDrawPage.BorderRight = nOldBorderBottom
    oDrawPage.BorderBottom = nOldBorderLeft


#------------------------------------------------------------
#   Shape functions
#------------------------------------------------------------
def PolyLineShape( oDrawDoc, oPosition=None, oSize=None,   aPoint=None):
    """Create a new RectangleShape with an optional position and size."""
    oShape = makeShape( oDrawDoc, "com.sun.star.drawing.PolyLineShape", oPosition, oSize )
    Points =  ListToArray(aPoints)
    oShape.PolyPolygon = Array( Points ) 
    return oShape
    
def makePolygonShape( oDrawDoc, oPosition=None, oSize=None,   aPoint=None):
    """Create a new RectangleShape with an optional position and size."""
    oShape = makeShape( oDrawDoc, "com.sun.star.drawing.PolyPolygonShape", oPosition, oSize )
    Points =  ListToArray(aPoints)
    oShape.PolyPolygon = Array( Points ) 
    return oShape
    
def makeTableShape( oDrawDoc, oPage,nRow,  nColumn, oPosition=None, oSize=None):
    """Create a new RectangleShape with an optional position and size."""
    oShape = makeShape( oDrawDoc, "com.sun.star.drawing.TableShape", oPosition, oSize )
    oPage.add(oShape)
    oShape.Model.getColumns().insertByIndex(0, nColumn-1)
    oShape.Model.getRows().insertByIndex(0, nRow-1)
    setTableFillColor(oShape.Model)#заливка по умолчанию белая
    setTableLineColor(oShape.Model)
    setTableColumnsWidth(oShape.Model, 2000)#ширина столбцов по умолчанию 2 см
    setTableRowsHigth(oShape.Model, 700)#высота строк по умолчанию 0.7см
    return oShape

def makePictureShape( oDrawDoc, oPosition=None, oSize=None ):
    """Create a new RectangleShape with an optional position and size."""
    oShape = makeShape( oDrawDoc, "com.sun.star.drawing.GraphicObjectShape", oPosition, oSize )
    return oShape

def makeRectangleShape( oDrawDoc, oPosition=None, oSize=None ):
    """Create a new RectangleShape with an optional position and size."""
    oShape = makeShape( oDrawDoc, "com.sun.star.drawing.RectangleShape", oPosition, oSize )
    return oShape

def makeEllipseShape( oDrawDoc, oPosition=None, oSize=None ):
    """Create a new EllipseShape with an optional position and size."""
    oShape = makeShape( oDrawDoc, "com.sun.star.drawing.EllipseShape", oPosition, oSize )
    return oShape

def makeLineShape( oDrawDoc, oPosition=None, oSize=None ):
    """Create a new LineShape with an optional position and size."""
    oShape = makeShape( oDrawDoc, "com.sun.star.drawing.LineShape", oPosition, oSize )
    return oShape

def makeTextShape( oDrawDoc, oPosition=None, oSize=None ):
    """Create a new TextShape with an optional position and size."""
    oShape = makeShape( oDrawDoc, "com.sun.star.drawing.TextShape", oPosition, oSize )
    return oShape

def findShapeByName( oShapes, cShapeName ):
    """Find a named shape within an XShapes interface.
    oShapes can be a drawing page, which supports the XShapes interface.
    Thus, you can find a named shape within a draw page, or within a grouped shape,
     or within a selection of sseveral shapes.
    """
    nNumShapes = oShapes.getCount()
    for i in range( nNumShapes ):
        oShape = oShapes.getByIndex( i )
        cTheShapeName = oShape.getName()
        if cTheShapeName == cShapeName:
            return oShape
    return None

def makeShape( oDrawDoc, cShapeClassName, oPosition=None, oSize=None ):
    """Create a new shape of the specified class.
    Position and size arguments are optional.
    """
    oShape = oDrawDoc.createInstance( cShapeClassName )

    if oPosition != None:
        oShape.Position = oPosition
    if oSize != None:
        oShape.Size = oSize
    return oShape

#------------------------------------------------------------
#   Color manipulation
#------------------------------------------------------------

def rgbColor( nRed, nGreen, nBlue ):
    """Return an integer which repsents a color.
    The color is specified in RGB notation.
    Each of nRed, nGreen and nBlue must be a number from 0 to 255.
    """
    return (int( nRed ) & 255) << 16 | (int( nGreen ) & 255) << 8 | (int( nBlue ) & 255)

def hsbColor( nHue, nSaturation, nBrightness ):
    """Return an integer which repsents a color.
    The color is specified in HSB notation.
    Each of nHue, nSaturation and nBrightness must be a number from 0.0 to 1.0.
    """
    nRed, nGreen, nBlue = HSBConversions.HSBtoRGB( nHue, nSaturation, nBrightness )
    return rgbColor( nRed, nGreen, nBlue )

def redColor( nColor ):
    """Return the Red component of a color as an integer from 0 to 255.
    nColor is an integer representing a color.
    This function is complimentary to the rgbColor function.
    """
    return (int( nColor ) >> 16) & 255

def greenColor( nColor ):
    """Return the Green component of a color as an integer from 0 to 255.
    nColor is an integer representing a color.
    This function is complimentary to the rgbColor function.
    """
    return (int( nColor ) >> 8) & 255

def blueColor( nColor ):
    """Return the Blue component of a color as an integer from 0 to 255.
    nColor is an integer representing a color.
    This function is complimentary to the rgbColor function.
    """
    return int( nColor ) & 255

#------------------------------------------------------------
#   Drawing routines
#------------------------------------------------------------
# Multiply this number by an OOo angle in 100'ths of a degree
#  to convert to radians.
nRadiansPerHundredthDegree = math.pi / 18000

def drawLine( oDrawDoc, oDrawPage, x1,y1, x2,y2, nLineColor=None ):
    """Draw a line from x1,y1 to x2,y2.  Optionally specify line color.
    This adds the LineShape to the page.
    The LineShape is returned.
    """
    # make sure size is non-zero
    #if x1 = x2: x2 = x1 + 1
    #if y1 = y2: y2 = y1 + 1

    oPosition = makePoint( x1, y1 )
    oSize = makeSize( x2-x1, y2-y1 )

    oShape = makeLineShape( oDrawDoc, oPosition, oSize )

    if nLineColor != None:
        oShape.LineColor = nLineColor
    #oShape.LineWidth = 0
    oDrawPage.add( oShape )
    return oShape

def drawLineVector( oDrawDoc, oDrawPage, x1,y1, nAngle,nDistance, nLineColor=None ):
    """Draw a line from x1,y1 in the direction of nAngle, for a distance of nDistance.
    nAngle is measured in radians, clockwise from the 3 O'Clock (east) direction.
    nDistance is in 1000ths of a centimeter.
    This adds the LineShape to the page.
    The LineShape is returned.
    """
    nDX = math.cos( nAngle ) * nDistance
    nDY = math.sin( nAngle ) * nDistance
    return drawLine( oDrawDoc, oDrawPage, x1,y1, x1+nDX,y1+nDY, nLineColor )

def drawAutoSizingText( oDrawDoc, oDrawPage,
                        cText, oPoint, rotateAngle=0, charHeight=14, charWeight=100, charColor=0,  nExtraWidthPercent=40 ):
    """Create a TextShape that will automatically resize its characters
     to its shape bounding rectangle.
    The TextShape is created with a specified height.
    The width is determined based on the natural character width of the
     text.
    The initial position of the text is -10000,-10000, so that the text is not visible.
    This returns the TextShape, which has already been added to the drawing page,
     at coordinates which make it invisible.
    You must set the object's Position property to make the text visible.
    (Hint: You may look at the Size property to help you determine where
     you want to place the text, for instance if you are trying to center
     it, or place it relative to some other shape object.)
    If you don't supply nExtraWidthPercent, then a default fudge factor is used.
    This is the percentage of the average character width, which is added to the shape's
     total width.
    """
    # Create TextShape
    size = makeSize( 1, 1 )     
    oShape = makeTextShape( oDrawDoc, oPoint, size)
    
    # Add it to the page.
    oDrawPage.add( oShape )

    # Make text stick to upper left corner of the shape rather than centered within the shape.
    oShape.TextHorizontalAdjust = uno.getConstantByName( "com.sun.star.drawing.TextHorizontalAdjust.LEFT" )
    oShape.TextVerticalAdjust = uno.getConstantByName( "com.sun.star.drawing.TextVerticalAdjust.TOP" )

    # Make the shape auto-grow in size, based on the text.
    # Once we set the text, in the next step, the shape will grow to some
    #  unknown size, based on the current font in use.
   
    oShape.TextAutoGrowHeight = True
    oShape.TextAutoGrowWidth = True

    
    # Set the text of the TextShape.
    # Because of the TextAutoGrowWidth, the shape now occupies some unknown size.
    text = oShape.Text
    cursor = text.createTextCursor()
    cursor.setPropertyValue("CharColor", charColor)
    cursor.setPropertyValue("CharHeight", charHeight)
    cursor.setPropertyValue("CharFontName", "Times New Roman")
    cursor.setPropertyValue("CharWeight", charWeight)
    text.insertString(cursor, cText,0)
    
    
    
    # Get the shape's current size.    
    nSaveHeight = oShape.Size.Height
    nSaveWidth = oShape.Size.Width

    # Make the shape NOT auto-grow in size, based on the text.
    #oShape.TextAutoGrowHeight = False
    #oShape.TextAutoGrowWidth = False

    # This next setting causes the TextShape to automatically resize its characters
    #  to fit the size of the text shape.
#    oShape.TextFitToSize = uno.getConstantByName( "com.sun.star.drawing.TextFitToSizeType.PROPORTIONAL" )
    # Calculate the new width, based on the desired height,
    #  and saved width/height ratio for the current font in use.
    
    lent = len( cText )
    if lent == 0:
        lent = 1
    nAverageCharacterWidth = nSaveWidth / lent

    nExtraWidth = nAverageCharacterWidth * (nExtraWidthPercent / 100.0)

    oShape.TextLeftDistance = 10
    oShape.TextRightDistance = 10
    oShape.TextUpperDistance = 10
    oShape.TextLowerDistance = 10
    
    nSaveWidth = nSaveWidth + 2 * nExtraWidth

    # Now resize the TextShape.
    #oShape.Size.Width = nSaveWidth
    #oShape.Size.Height = nSaveHeight  
    oShape.RotateAngle = rotateAngle * 100  
    oShape.Position = oPoint
    return oShape


def drawArcPath( oDrawDoc, oDrawPage,
                 nStartX, nStartY, nStartAngle,
                 nArcAngle, nArcRadius, bTurnLeft ):
    """Draw an arc of a circle from a starting position and direction.
    The arc has a certian radius and arc angle,
     and turns either to the left or to the right.
    Parameters:
    nStartX and nStartY are the starting position of the "turtle".
    nStartAngle is the angle direction that the turtle is pointing,
     in 100'ths of a degree.
    An arc is drawn by moving the turtle forward and turning either left or right as it moves.
    nArcAngle is the angle of the arc describing the turtle's path,
     in 100'ths of a degree.
    nArcRadius is the radius of the arc describing the turtle's path.
    bTurnLeft is True if the turtle turns to the left, or False to turn to the right.
    Four values are returned.
    (1) the circle shape, (2) the new X position, (3) new Y position, and (4) new angle.
    Use the last three return values as initial values to draw another arc
     which is connected to the ending position of the arc just drawn.
    """
    # Figure out how big of a circle that the arc is a part of.
    nCircleDiameter = nArcRadius + nArcRadius
    oCircleSize = makeSize( nCircleDiameter, nCircleDiameter )

    # Figure out the position of the circle (ellipse) shape object.
    #
    # Determine the angle of the center point from the starting position.
    if bTurnLeft:
        nCenterPointAngle = nStartAngle + 9000
    else:
        nCenterPointAngle = nStartAngle - 9000
    # Convert to radians.
    nCenterPointAngle = nCenterPointAngle * nRadiansPerHundredthDegree
    # Determine where the center of the circle shape should be.
    nCenterX = nStartX + (nArcRadius * math.cos( nCenterPointAngle ))
    nCenterY = nStartY - (nArcRadius * math.sin( nCenterPointAngle ))
    oCirclePosition = makePoint( nCenterX - nArcRadius, nCenterY - nArcRadius )

    # Figure out what arc portion of the circle needs to be drawn.
    # Angles measures in 100'ths of a degree.
    if bTurnLeft:
        nCircleStartAngle = nStartAngle - 9000
        nCircleEndAngle = nCircleStartAngle + nArcAngle
    else:
        nCircleEndAngle = nStartAngle + 9000
        nCircleStartAngle = nCircleEndAngle - nArcAngle

    # Make the circle shape.
    oCircle = makeEllipseShape( oDrawDoc, oCirclePosition, oCircleSize )
    # Fill in its properties
    oCircle.FillStyle = uno.getConstantByName( "com.sun.star.drawing.FillStyle.NONE" )
    oCircle.CircleKind = uno.getConstantByName( "com.sun.star.drawing.CircleKind.ARC" )
    oCircle.CircleStartAngle = nCircleStartAngle
    oCircle.CircleEndAngle = nCircleEndAngle
    # Put it on the drawing
    oDrawPage.add( oCircle )

    # Figure out the ending turtle location and direction.
    if bTurnLeft:
        nEndAngle = nStartAngle - 9000 + nArcAngle
    else:
        nEndAngle = nStartAngle + 9000 - nArcAngle
    # Convert to radians
    nEndAngleRad = nEndAngle * nRadiansPerHundredthDegree
    # Ending Position
    nEndX = nCenterX + (nArcRadius * math.cos( nEndAngleRad ))
    nEndY = nCenterY - (nArcRadius * math.sin( nEndAngleRad ))
    # Ending angle
    if bTurnLeft:
        nEndAngle = normalizeOOoAngle( nEndAngle + 9000 )
    else:
        nEndAngle = normalizeOOoAngle( nEndAngle - 9000 )

    return oCircle, nEndX, nEndY, nEndAngle


def drawSpiralOfArcs( oDrawDoc, oDrawPage,
                      nStartX, nStartY, nStartAngle,
                      nArcAngle, nArcRadius, bTurnLeft,
                      nNumArcs,
                      nRadiusGrowthMultiplier=1.0, nRadiusGrowthAddIn=0 ):
    """Draw a spiral starting from a certian position and direction.
    The spiral turns either towards the left or towards the right.
    Parameters:
    nStartX and nStartY are the starting position of the "turtle".
    nStartAngle is the angle direction that the turtle is pointing,
     in 100'ths of a degree.
    A spiral is drawn by moving the turtle forward and turning either left or right as it moves.
    nArcAngle is the angle of the first arc of the spiral,
     in 100'ths of a degree.
    nArcRadius is the radius of the first arc of the spiral.
    bTurnLeft is True if the turtle turns to the left, or False to turn to the right.
    nNumArcs is the number of arcs which are drawn.
    nRadiusGrowthMultiplier - the radius of the arc is multiplied by this after each arc is drawn.
     Use a number such as 1.1 to cause the radius to grow geometrically as the spiral turns.
    nRadiusGrowthAddIn - this is added to the radius after each arc.
     Use a number, such as the original nArcRadius / number of arcs in a complete circle to cause the radius to grow arithmeteically as the spiral turns.
    Four values are returned.
    (1) the spiral shape, (2) the new X position, (3) new Y position, and (4) new angle.
    Use the last three return values as initial values to draw another arc
     which is connected to the ending position of the arc just drawn.
    """
    nX = nStartX
    nY = nStartY
    nAngle = nStartAngle

    oShapesToGroup = createUnoService( "com.sun.star.drawing.ShapeCollection" )
    for i in range( nNumArcs - 1 ):
        oArcShape, nX, nY, nAngle = drawArcPath( oDrawDoc, oDrawPage, nX, nY, nAngle, nArcAngle, nArcRadius, bTurnLeft )
        nArcRadius = nArcRadius * nRadiusGrowthMultiplier + nRadiusGrowthAddIn

        oShapesToGroup.add( oArcShape )
    oSpiralShape = oDrawPage.group( oShapesToGroup )
    return oSpiralShape, nX, nY, nAngle

def createShapeCollection(oDrawPage, oShape):
    oShapesToGroup = createUnoService( "com.sun.star.drawing.ShapeCollection" )
    oShapesToGroup.add(oShape)
    oShapesToGroup = oDrawPage.group( oShapesToGroup )
    return oShapesToGroup
    
#------------------------------------------------------------
#   Styles
#------------------------------------------------------------
def defineGraphicsStyle( oDrawDoc, cStyleName, cParentStyleName=None ):
    """Add a new style to the style catalog if it is not already present.
    This returns the style object so that you can alter its properties.
    """
    return defineStyle( oDrawDoc, "graphics", cStyleName, cParentStyleName )


def getGraphicsStyle( oDrawDoc, cStyleName ):
    """Lookup and return a graphics style from the document.
    """
    return getStyle( oDrawDoc, "graphics", cStyleName )
    
#------------------------------------------------------------
#   General Utility functions
#------------------------------------------------------------
def normalizeOOoAngle( nAngleOOo ):
    """Given an angle in 100'ths of a degree,
    adjust it to be from 0 to 360 degrees."""
    if nAngleOOo < 0:
        nSign = -1
    else:
        nSign = 1
    nAngleOOo = nAngleOOo - (int( abs( nAngleOOo ) / 36000.0 ) * 36000 * nSign)
    if nAngleOOo < 0:
        nAngleOOo += 36000
    return nAngleOOo  

#устанавливает цвет заливки области
def setFillColor(oShape,  color):
    oShape.FillColor= color

#устанавливает цвет линий
def setLineColor(oShape,  color):
    oShape.LineColor= color

#вставка текста в ячейку таблицы
def insertTextIntoCell(table, cellRow,  cellColumn, text, color, rotation, alignment):
    cell = table.getCellByPosition(cellColumn,  cellRow)
    cursor = cell.createTextCursor()
    cursor.setPropertyValue("CharColor", color)
    #        cursor.setPropertyValue("CharRotation",rotation)
    cursor.setPropertyValue("ParaAdjust",alignment);
    cursor.setPropertyValue("CharFontName","Times New Roman");#шрифт по умолчанию
    cursor.setPropertyValue("CharHeight",14);#размер шрифта по умолчанию
    cell.setString(text.decode("utf-8"))

#ширина столбцов
def setTableColumnsWidth(table,  width):
    columns = table.getColumns()
    num = columns.getCount()
    for i in range(num):
        column = columns.getByIndex(i)
        column.setPropertyValue("Width", width)
            
#высота строк
def setTableRowsHigth(table,  height):
    rows = table.getRows()
    num = rows.getCount()
    for i in range(num):
        row = rows.getByIndex(i)
        row.setPropertyValue("Height", height)

#устанавливает цвет заливки таблицы
def setTableFillColor(oTable,  color = None):
    if color == None:
        color = rgbColor(255, 255, 255)
    nRow = oTable.getRows().getCount()
    nColumn = oTable.getColumns().getCount()
    for i in range(nRow):
       for j in range(nColumn):
            oTable.getCellByPosition(j, i).setPropertyValue("FillColor",color);
def setTableLineColor(oTable,  color = None):
    if color == None:
        color = rgbColor(0, 0, 0)
    aLine = createUnoStruct("com.sun.star.table.BorderLine")
    aLine.Color = color
    aLine.InnerLineWidth = aLine.LineDistance = 10;
    aLine.OuterLineWidth = 10;
    nRow = oTable.getRows().getCount()
    nColumn = oTable.getColumns().getCount()
    for i in range(nRow):
       for j in range(nColumn):
            oTable.getCellByPosition(j, i).RightBorder = oTable.getCellByPosition(j, i).LeftBorder =  oTable.getCellByPosition(j, i).TopBorder = oTable.getCellByPosition(j, i).BottomBorder = aLine
############################################################################################################

getServiceManager()
oDesktop = getDesktop()

doc =  oDesktop.loadComponentFromURL('private:factory/simpress', '_blank', 0, ()) #StarDesktop.getCurrentComponent()
pages = doc.getDrawPages()
target = pages.getByIndex(0)

pages = doc.DrawPages
target = pages.getByIndex(0)
pages = doc.DrawPages
target = pages.getByIndex(0)
